/***************************************************************************
                          csvimportdlg.cpp  -  description
                             -------------------
    begin                : Don Aug 21 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "csvimportdlg.h"

// Qt includes
#include <qfile.h>
#include <qframe.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qsqlquery.h>
#include <qtable.h>
#include <qtextstream.h>
#include <qsqldatabase.h>

// KDE includes
#include <kaction.h>
#include <kapplication.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kpopupmenu.h>
#include <kpushbutton.h>
#include <kurlrequester.h>
#include <kstatusbar.h>

// a small helper function
QString removeQuote( QString text, QString quote )
{
    if( quote.isEmpty() )
        return text;
        
    text = text.stripWhiteSpace();

    if( text.left( quote.length() ) == quote )
        text = text.mid( quote.length(), text.length() - quote.length() );

    if( text.right( quote.length() ) == quote )
        text = text.left( text.length() - quote.length() );

    return text;
}

const char* NOFIELD = "<NONE>";

CSVImportDlg::CSVImportDlg( QSqlDatabase* database)
    : KMainWindow()
{
    db = database;

    KAction* quitAct = KStdAction::quit(kapp, SLOT(quit()), actionCollection());

    KPopupMenu* mnuFile = new KPopupMenu( this );
    quitAct->plug( mnuFile );

    menuBar()->insertItem( i18n("&File"), mnuFile );
    menuBar()->insertItem( i18n("&Help"), helpMenu() );

    QFrame* box = new QFrame( this );
    box->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    setCentralWidget( box );

    QVBoxLayout* layout = new QVBoxLayout( box, 6, 6 );

    requester = new KURLRequester( box );

    comboSQL = new KComboBox( false, box );
    QSqlQuery q( "SHOW TABLES" );
    while( q.next() )
        comboSQL->insertItem( q.value( 0 ).toString() );

    table = new QTable( box );
    table->setReadOnly( true );

    frame = new QFrame( box );
    QHBoxLayout* layout2 = new QHBoxLayout( frame, 6, 6 );
    QHBoxLayout* layout3 = new QHBoxLayout( 0, 6, 6 );

    spinCol = new KIntNumInput( frame );
    spinCol->setLabel( i18n("Column:"), AlignLeft | AlignVCenter );
    spinCol->setRange( 0, 0, 0, false );

    comboField = new KComboBox( false, frame );
    buttonSet = new KPushButton( i18n("Set"), frame );

    buttonOk = new KPushButton( i18n("&Import"), box );

    layout2->addWidget( spinCol );
    layout2->addWidget( new QLabel( i18n("Database field to use for this column:"), frame ) );
    layout2->addWidget( comboField );
    layout2->addWidget( buttonSet );

    QSpacerItem* spacer = new QSpacerItem( 0, 0, QSizePolicy::Expanding,QSizePolicy::Minimum );
    layout3->addItem( spacer );
    layout3->addWidget( buttonOk );

    statusBar()->insertFixedItem( i18n("Selected Database: <b>" ) + db->databaseName() + "</b>", 0, true );
    statusBar()->message( i18n("Select a CSV file first, before you can proceede."), 2000 );

    layout->addWidget( new QLabel( i18n("File to import:"), box ) );
    layout->addWidget( requester );
    layout->addWidget( new QLabel( i18n("Import into table:"), box ) );
    layout->addWidget( comboSQL );
    layout->addWidget( table );
    layout->setStretchFactor( table, 2 );
    layout->addWidget( frame );
    layout->addLayout( layout3 );

    connect( requester, SIGNAL( textChanged( const QString & ) ), this, SLOT( settingsChanged() ) );
    connect( buttonSet, SIGNAL( clicked() ), this, SLOT( setCol() ) );
    connect( table->horizontalHeader(), SIGNAL( clicked( int ) ), this, SLOT( updateCol( int ) ) );
    connect( comboSQL, SIGNAL( activated(int)), this, SLOT( updateFields() ) );
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( import() ) );
    updateFields();
    enableControls();

    loadConfig();
    show();
}

CSVImportDlg::~CSVImportDlg()
{
    QString m_database = db->databaseName();

    db->close();
    QSqlDatabase::removeDatabase( m_database );

    saveConfig();
}

void CSVImportDlg::loadConfig()
{
    KConfig* config = kapp->config();

    comment = config->readEntry( "comment", "#" );
    separator = config->readEntry( "separator", ";" );
    quote = config->readEntry( "quote", "\"" );

    restoreWindowSize( config );
}

void CSVImportDlg::saveConfig()
{
    KConfig* config = kapp->config();

    config->writeEntry( "comment", comment );
    config->writeEntry( "separator", separator );
    config->writeEntry( "quote", quote );

    saveWindowSize( config );

    config->sync();
}

void CSVImportDlg::settingsChanged()
{
    QFile file( requester->url() );
    if( !file.open( IO_ReadOnly ) ) {
        table->setNumCols( 0 );
        table->setNumRows( 0 );
        return;
    }

    QTextStream t( &file );
    table->setNumCols( 1 );
    int i = 0;
    while( !t.atEnd() ) {
        QString s = t.readLine();

        // same as in labelprinter.cpp
        s = s.stripWhiteSpace();
        if( s.left( comment.length() ) == comment && !comment.isEmpty() )
            /* Line is a comment */
            continue;

        if( s.isEmpty() )
            continue;

        // calculate the number of rows
        int c = s.contains( separator );
        if( table->numCols() < c )
            table->setNumCols( c );

        if( table->numRows() <= i )
            // add 100 rows to get a reasonable speed
            table->setNumRows( i + 100 );

        for( int z = 0; z < c; z++ )
            table->setText( i, z, removeQuote( s.section( separator, z, z ), quote ) );
        i++;
    }

    table->setNumRows( i );
    file.close();

    spinCol->setRange( 1, table->numCols(), 1, false );
        
    enableControls();
}

void CSVImportDlg::setCol()
{
    QString text = comboField->currentText();
    int v = spinCol->value() - 1;
    if( text == NOFIELD )
        table->horizontalHeader()->setLabel( v, QString::number( v + 1 ) );
    else {
        for( int i = 0; i < table->horizontalHeader()->count(); i++ )
            if( table->horizontalHeader()->label( i ) == text )
                table->horizontalHeader()->setLabel( i, QString::number( i + 1 ) 
);                
        table->horizontalHeader()->setLabel( v, text );
    }

    enableControls();
}

void CSVImportDlg::updateFields()
{
    comboField->clear();
    comboField->insertItem( NOFIELD );
    QSqlQuery query( "SHOW FIELDS FROM " + comboSQL->currentText() );
    while( query.next() )
        comboField->insertItem( query.value( 0 ).toString() );

    for( int i = 0; i < table->horizontalHeader()->count(); i++ )
        table->horizontalHeader()->setLabel( i, QString::number( i + 1 ) );

    enableControls();
}

void CSVImportDlg::enableControls()
{
    int c = 0;
    // be sure that the user asigned a database field
    // to at least one column of the CSV file
    for( int i = 0; i < table->horizontalHeader()->count(); i++ )
        if( table->horizontalHeader()->label( i ) != QString::number( i + 1 ) )
            c++;

    bool b = table->numRows() && table->numCols() && comboSQL->count();

    buttonOk->setEnabled( b && c );
    frame->setEnabled( b );
}

void CSVImportDlg::updateCol( int c )
{
    spinCol->setValue( ++c );
}

void CSVImportDlg::import()
{
    QHeader* h = table->horizontalHeader();
    QValueList<int> headers;
        
    QString q = "INSERT INTO " + comboSQL->currentText() + " (";
    for( int c = 0; c < table->horizontalHeader()->count(); c++ ) {
        bool ok = true;
        h->label( c ).toInt( &ok );
        if( !ok ) {
            q = q + table->horizontalHeader()->label( c ) + ",";
            headers << c;
        }
    }

    // remove last ","
    if( q.right( 1 ) == "," )
        q = q.left( q.length() - 1 );

    q = q + ") VALUES (";

    for( int i = 0; i < table->numRows(); i++ ) {
        QString line = q;
        for( unsigned int c = 0; c < headers.count(); c++ )
            line.append( "'" + table->text( i, headers[c] ) + "'" + "," );

        // remove last ","
        if( line.right( 1 ) == "," )
            line = line.left( line.length() - 1 );

        line = line + ");";

        QSqlQuery query;
        if( !query.exec( line ) )
            KMessageBox::error( this, i18n("Could not import the following line:") + line );
    }

    KMessageBox::information( this, i18n("Data was imported successfully.") );

    // No clear the UI
    requester->setURL( "" );
    settingsChanged();
    enableControls();
}


